from __future__ import annotations

from typing import Any, Callable

from werkzeug.utils import import_string

import ckan.plugins.toolkit as tk

ROUTE_PATH = "create_typed_package.route_path"
USE_SEPARATE_ROUTE = "create_typed_package.use_separate_route"

USE_SCHEMING = "create_typed_package.use_scheming"
ADDITIONAL_TYPES = "create_typed_package.additional_types"
EXCLUDE_TYPES = "create_typed_package.exclude_types"
SORTER = "create_typed_package.sorter"


def route_path() -> str:
    return tk.config[ROUTE_PATH]


def use_separate_route() -> bool:
    return tk.config[USE_SEPARATE_ROUTE]


def use_scheming() -> bool:
    return tk.config[USE_SCHEMING]


def additional_types() -> list[str]:
    return tk.config[ADDITIONAL_TYPES]


def exclude_types() -> list[str]:
    return tk.config[EXCLUDE_TYPES]


def sorter() -> Callable[[Any], Any]:
    return import_string(tk.config[SORTER])



import ckantoolkit as tk
import pytest

import ckan.model as model
import ckan.tests.factories as factories
import ckan.tests.helpers as helpers


@pytest.mark.usefixtures("clean_db", "app", "with_plugins")
def test_ctp_list_types():
    context = {"model": model, "user": ""}
    with pytest.raises(tk.NotAuthorized):
        helpers.call_auth("ctp_list_types", context)

    user = factories.User()
    context = {"model": model, "user": user["name"]}
    assert helpers.call_auth("ctp_list_types", context)



from __future__ import annotations

import ckan.plugins.toolkit as tk
from ckan import plugins as p

from ckanext.create_typed_package import helpers, views
from ckanext.create_typed_package.logic import action, auth


@tk.blanket.config_declarations
class CreateTypedPackagePlugin(p.SingletonPlugin):
    p.implements(p.IConfigurer)
    p.implements(p.IActions)
    p.implements(p.IAuthFunctions)
    p.implements(p.IBlueprint)
    p.implements(p.ITemplateHelpers)

    # IConfigurer

    def update_config(self, config_):
        tk.add_template_directory(config_, "templates")
        tk.add_resource("assets", "create_typed_package")

    # IActions

    def get_actions(self):
        return action.get_actions()

    # IAuthFunctions

    def get_auth_functions(self):
        return auth.get_auth_functions()

    # IBlueprint

    def get_blueprint(self):
        if tk.h.ctp_use_separate_route():
            return views.get_blueprints()
        return []

    # ITemplateHelpers

    def get_helpers(self):
        return helpers.get_helpers()



from __future__ import annotations

import ckan.plugins.toolkit as tk

from . import config


def get_helpers():
    return {
        "ctp_use_separate_route": ctp_use_separate_route,
        "ctp_available_types": ctp_available_types,
    }


def ctp_use_separate_route():
    return config.use_separate_route()


def ctp_available_types():
    return tk.get_action("ctp_list_types")({}, {"with_labels": True})



from __future__ import annotations

from operator import itemgetter

from typing import Any
from ckan import types
import ckan.plugins as p
import ckan.plugins.toolkit as tk
import ckan.lib.plugins as lib_plugins
from ckanext.create_typed_package import config

CONFIG_LABEL_PREFIX = "create_typed_package.label_for."

default_sorter = itemgetter("label")


def get_actions() -> dict[str, Any]:
    return {
        "ctp_list_types": ctp_list_types,
    }


def _labels_from_config() -> dict[str, str]:
    labels: dict[str, str] = {}
    for option, value in tk.config.items():
        if not option.startswith(CONFIG_LABEL_PREFIX):
            continue
        labels[option[len(CONFIG_LABEL_PREFIX) :]] = value
    return labels


@tk.side_effect_free
def ctp_list_types(context: types.Context, data_dict: dict[str, Any]):
    with_labels = tk.asbool(data_dict.get("with_labels"))

    tk.check_access("ctp_list_types", context, data_dict)

    dt = _get_scheming_types() if _use_scheming() else _get_native_types()
    result = list(set(dt).union(_additional_types()).difference(_exclude_types()))

    if with_labels:
        labels = _labels_from_config()
        sorter = config.sorter()
        result = sorted(
            [{"name": t, "label": labels.get(t) or tk._(t)} for t in result],
            key=sorter,
        )
    return result


def _get_native_types():
    return list(lib_plugins._package_plugins) + ["dataset"]  # pyright: ignore[reportPrivateUsage]


def _get_scheming_types() -> list[str]:
    if not p.plugin_loaded("scheming_datasets"):
        return []
    return tk.get_action("scheming_dataset_schema_list")({}, {})


def _use_scheming():
    return config.use_scheming()


def _additional_types():
    return config.additional_types()


def _exclude_types():
    return config.exclude_types()



import ckantoolkit as tk


def get_auth_functions():
    return {
        "ctp_list_types": ctp_list_types,
    }


@tk.auth_allow_anonymous_access
def ctp_list_types(context, data_dict=None):
    tk.check_access("package_create", context, data_dict)
    return {"success": True}



from __future__ import annotations

from flask import Blueprint
from flask.views import MethodView

import ckan.plugins.toolkit as tk

from . import config


def get_blueprints():
    ctp = Blueprint("create_typed_package", __name__)
    path = config.route_path()

    ctp.add_url_rule(path, view_func=SelectDatasetTypeView.as_view("select_dataset_type"))
    return [ctp]


class SelectDatasetTypeView(MethodView):
    def post(self):
        type_ = tk.request.form["type"]
        return tk.redirect_to(type_ + ".new")

    def get(self):
        types = tk.get_action("ctp_list_types")({"user": tk.c.user}, {"with_labels": True})
        extra_vars = {
            "form_snippet": "package/snippets/ctp_select_dataset_type_form.html",
            "pkg_dict": {},
            "form_vars": {"package_types": [{"value": t["name"], "text": t["label"]} for t in types]},
        }
        return tk.render("package/ctp_select_dataset_type.html", extra_vars)



import pytest

import ckan.tests.factories as factories


@pytest.mark.usefixtures("with_plugins")
def test_route_not_available(app):
    app.get("/dataset/select-type", status=404)


@pytest.mark.ckan_config("create_typed_package.use_separate_route", "yes")
@pytest.mark.usefixtures("clean_db", "with_plugins")
def test_route_available(app):
    user = factories.User()
    env = {"REMOTE_USER": user["name"]}
    app.get("/dataset/select-type", environ_overrides=env, status=200)


@pytest.mark.ckan_config("create_typed_package.use_separate_route", "yes")
@pytest.mark.ckan_config("create_typed_package.route_path", "/ctp/route/path")
@pytest.mark.usefixtures("clean_db", "with_plugins")
def test_route_available_under_custom_path(app):
    user = factories.User()
    env = {"REMOTE_USER": user["name"]}
    app.get("/dataset/select-type", environ_overrides=env, status=404)
    app.get("/ctp/route/path", environ_overrides=env, status=200)



import pytest

import ckan.tests.helpers as helpers


@pytest.mark.usefixtures("with_plugins")
class TestCtpListTypes:
    def test_ctp_list_types_default(self):
        types = helpers.call_action("ctp_list_types")
        assert types == ["dataset"]

    @pytest.mark.ckan_config("create_typed_package.additional_types", "first second")
    def test_ctp_list_types_include(self):
        types = helpers.call_action("ctp_list_types")
        assert sorted(types) == ["dataset", "first", "second"]

    @pytest.mark.ckan_config("create_typed_package.additional_types", "first second")
    def test_ctp_list_types_with_default_labels(self):
        types = helpers.call_action("ctp_list_types", with_labels=True)
        assert types == [
            {"name": "dataset", "label": "dataset"},
            {"name": "first", "label": "first"},
            {"name": "second", "label": "second"},
        ]

    @pytest.mark.ckan_config("create_typed_package.label_for.dataset", "Dataset")
    @pytest.mark.ckan_config("create_typed_package.label_for.second", "#2 Dataset")
    @pytest.mark.ckan_config("create_typed_package.additional_types", "first second")
    def test_ctp_list_types_with_custom_labels(self):
        types = helpers.call_action("ctp_list_types", with_labels=True)
        assert types == [
            {"name": "second", "label": "#2 Dataset"},
            {"name": "dataset", "label": "Dataset"},
            {"name": "first", "label": "first"},
        ]

    @pytest.mark.ckan_config("create_typed_package.exclude_types", "dataset")
    def test_ctp_list_types_exclude(self):
        types = helpers.call_action("ctp_list_types")
        assert types == []

    @pytest.mark.ckan_config("ckan.plugins", "create_typed_package example_idatasetform_v6")
    def test_ctp_list_types_idatasetform_v6(self):
        types = helpers.call_action("ctp_list_types")
        assert sorted(types) == ["dataset", "fancy_type"]

    @pytest.mark.ckan_config(
        "ckan.plugins",
        "create_typed_package example_idatasetform_v6 scheming_datasets",
    )
    def test_ctp_list_types_idatasetform_v6_not_using_scheming(self):
        types = helpers.call_action("ctp_list_types")
        assert sorted(types) == ["dataset", "fancy_type"]

    @pytest.mark.ckan_config(
        "ckan.plugins",
        "create_typed_package example_idatasetform_v6 scheming_datasets",
    )
    @pytest.mark.ckan_config("create_typed_package.use_scheming", "true")
    def test_ctp_list_types_idatasetform_v6_using_scheming(self):
        types = helpers.call_action("ctp_list_types")
        assert types == []

    @pytest.mark.ckan_config("create_typed_package.use_scheming", "true")
    def test_ctp_list_types_using_disabled_scheming(self):
        types = helpers.call_action("ctp_list_types")
        assert types == []

    @pytest.mark.ckan_config("ckan.plugins", "create_typed_package scheming_datasets")
    @pytest.mark.ckan_config("scheming.dataset_schemas", "ckanext.scheming:ckan_dataset.yaml")
    @pytest.mark.ckan_config("create_typed_package.use_scheming", "true")
    def test_ctp_list_types_using_one_schema(self):
        types = helpers.call_action("ctp_list_types")
        assert types == ["dataset"]

    @pytest.mark.ckan_config("ckan.plugins", "create_typed_package scheming_datasets")
    @pytest.mark.ckan_config(
        "scheming.dataset_schemas",
        "ckanext.scheming:ckan_dataset.yaml ckanext.scheming:camel_photos.yaml",
    )
    @pytest.mark.ckan_config("create_typed_package.use_scheming", "true")
    def test_ctp_list_types_using_two_schemas(self):
        types = helpers.call_action("ctp_list_types")
        assert sorted(types) == ["camel-photos", "dataset"]

    @pytest.mark.ckan_config(
        "scheming.dataset_schemas",
        "ckanext.scheming:ckan_dataset.yaml ckanext.scheming:camel_photos.yaml",
    )
    @pytest.mark.ckan_config("ckan.plugins", "create_typed_package scheming_datasets")
    def test_ctp_list_types_using_two_schemas_scheming_disabled(self, app):
        types = helpers.call_action("ctp_list_types")
        assert sorted(types) == ["camel-photos", "dataset"]

    @pytest.mark.ckan_config(
        "ckan.plugins",
        "create_typed_package scheming_datasets example_idatasetform_v6",
    )
    @pytest.mark.ckan_config(
        "scheming.dataset_schemas",
        "ckanext.scheming:ckan_dataset.yaml ckanext.scheming:camel_photos.yaml",
    )
    def test_ctp_list_types_using_idatasetform_v6_two_schemas_not_scheming(
        self,
    ):
        types = helpers.call_action("ctp_list_types")
        assert sorted(types) == ["camel-photos", "dataset", "fancy_type"]

    @pytest.mark.ckan_config(
        "ckan.plugins",
        "create_typed_package scheming_datasets example_idatasetform_v6",
    )
    @pytest.mark.ckan_config(
        "scheming.dataset_schemas",
        "ckanext.scheming:ckan_dataset.yaml ckanext.scheming:camel_photos.yaml",
    )
    @pytest.mark.ckan_config("create_typed_package.use_scheming", "true")
    def test_ctp_list_types_using_idatasetform_v6_two_schemas_scheming(
        self,
    ):
        types = helpers.call_action("ctp_list_types")
        assert sorted(types) == ["camel-photos", "dataset"]
